@@ -99,7 +99,7 @@ class Sentry(object):
                  logging=False, logging_exclusions=None, level=logging.NOTSET,
                  wrap_wsgi=None, register_signal=True):
         if client and not isinstance(client, Client):
-            raise TypeError('client should an instance of Client')
+            raise TypeError('client should be an instance of Client')
 
         self.dsn = dsn
         self.logging = logging
@@ -216,6 +216,10 @@ class Sentry(object):
 
     def before_request(self, *args, **kwargs):
         self.last_event_id = None
+
+        if request.url_rule:
+            self.client.transaction.push(request.url_rule.rule)
+
         try:
             self.client.http_context(self.get_http_info(request))
         except Exception as e:
@@ -229,6 +233,8 @@ class Sentry(object):
         if self.last_event_id:
             response.headers['X-Sentry-ID'] = self.last_event_id
         self.client.context.clear()
+        if request.url_rule:
+            self.client.transaction.pop(request.url_rule.rule)
         return response
 
     def init_app(self, app, dsn=None, logging=None, level=None,
